@@ -1,19 +1,20 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2011-2012 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2011 -- leonerd@leonerd.org.uk
 
 package Tickit::OneLineWidget;
 
 use strict;
 use warnings;
 use base qw( Tickit::Widget );
-use Tickit::WidgetRole::Alignable name => 'valign', dir => 'v';
 
-our $VERSION = '0.29_002';
+our $VERSION = '0.06';
 
 use Carp;
 
+use Tickit::Utils qw( align );
+
 =head1 NAME
 
 C<Tickit::OneLineWidget> - a widget which occupies only one line
@@ -56,6 +57,14 @@ sub lines
 
 =head2 $valign = $widget->valign
 
+=cut
+
+sub valign
+{
+   my $self = shift;
+   return $self->{valign};
+}
+
 =head2 $widget->set_valign( $valign )
 
 Accessor for vertical alignment value.
@@ -65,29 +74,43 @@ within the window. If the window is taller than one line, it will be aligned
 according to this value; with C<0.0> at the top, C<1.0> at the bottom, and
 other values inbetween.
 
-See also L<Tickit::WidgetRole::Alignable>
+The symbolic values C<top>, C<middle> and C<bottom> can be supplied instead of
+C<0.0>, C<0.5> and C<1.0> respectively.
 
 =cut
 
+sub set_valign
+{
+   my $self = shift;
+   my ( $valign ) = @_;
+
+   # Convert symbolics
+   $valign = 0.0 if $valign eq "top";
+   $valign = 0.5 if $valign eq "middle";
+   $valign = 1.0 if $valign eq "bottom";
+
+   $self->{valign} = $valign;
+
+   $self->redraw;
+}
+
 sub render
 {
    my $self = shift;
-   my %args = @_;
 
-   my $win = $self->window or return;
-   $win->is_visible or return;
-   my $rect = $args{rect};
+   my $window = $self->window or return;
 
-   my ( $above ) = $self->_valign_allocation( 1, $win->lines );
+   my ( $top ) =
+      Tickit::Utils::align( 1, $window->lines, $self->valign );
 
-   my $cols = $win->cols;
+   my $cols = $window->cols;
 
-   $win->goto( $_, 0 ), $win->erasech( $cols ) for $rect->top .. $above - 1;
+   $window->goto( $_, 0 ), $window->erasech( $cols ) for 0 .. $top - 1;
 
-   $win->goto( $above, 0 );
+   $window->goto( $top, 0 );
    $self->render_line;
 
-   $win->goto( $_, 0 ), $win->erasech( $cols ) for $above + 1 .. $rect->bottom - 1;
+   $window->goto( $_, 0 ), $window->erasech( $cols ) for $top + 1 .. $window->lines - 1;
 }
 
 =head1 SUBCLASS METHODS
